<?php

/**
 * @file
 * Tracks usage of embedded files.
 */

/**
 * Records file usage of files referenced by formatted text fields.
 *
 * Every referenced file that does not yet have the FILE_STATUS_PERMANENT state,
 * will be given that state.
 *
 * @param array $fids
 *   An array of file entity FIDs.
 * @param $entity
 *   An entity whose fields to inspect for file references.
 * @param $entity_type
 *   The type of the entity being inspected (i.e. node, user, comment).
 */
function _editor_record_file_usage(array $fids, $entity, $entity_type) {
  list($id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);

  foreach ($fids as $fid) {
    if ($file = file_load($fid)) {
      if ($file->status !== FILE_STATUS_PERMANENT) {
        $file->status = FILE_STATUS_PERMANENT;
        file_save($file);
      }
      file_usage_add($file, 'editor', $entity_type, $id);
    }
  }
}

/**
 * Deletes file usage of files referenced by formatted text fields.
 *
 * @param array $fids
 *   An array of file entity FIDs.
 * @param $entity
 *   An entity whose fields to inspect for file references.
 * @param $entity_type
 *    The type of the entity being inspected (i.e. node, user, comment).
 * @param $count
 *   The number of references to delete. Should be 1 when deleting a single
 *   revision and 0 when deleting an entity entirely.
 *
 * @see file_usage_delete()
 */
function _editor_delete_file_usage(array $fids, $entity, $entity_type, $count) {
  list($id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);

  foreach ($fids as $fid) {
    if ($file = file_load($fid)) {
      file_usage_delete($file, 'editor', $entity_type, $id, $count);
    }
  }
}

/**
 * Parse file references from an entity's text fields.
 *
 * @param string $entity_type
 *   The type of the entity whose fields are being parsed.
 * @param object $entity
 *   The entity whose fields are being parsed.
 *
 * @return array
 *   An array of file IDs corresponding to the files referenced in the fields.
 */
function _editor_get_file_fids_by_field($entity_type, $entity) {
  $fields = field_info_fields();
  $fids = array();

  $formatted_text_fields = _editor_get_formatted_text_fields($entity_type, $entity);
  foreach ($formatted_text_fields as $formatted_text_field) {
    $text = '';
    if ($field_items = field_get_items($entity_type, $entity, $formatted_text_field)) {
      foreach ($field_items as $field_item) {
        $text .= $field_item['value'];
        if ($fields[$formatted_text_field]['type'] == 'text_with_summary') {
          $text .= isset($field_item['summary']) ? $field_item['summary'] : '';
        }
      }
    }
    $fids[$formatted_text_field] = _editor_parse_file_fids($text);
  }

  return $fids;
}

/**
 * Determines the formatted text fields on an entity.
 *
 * @param $entity
 *   An entity whose fields to analyze.
 *
 * @return array
 *   The names of the fields on this entity that support formatted text.
 */
function _editor_get_formatted_text_fields($entity_type, $entity) {
  $entity_info = entity_get_info($entity_type);
  if (empty($entity_info['fieldable'])) {
    return array();
  }

  list($id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);

  $fields = field_info_fields();

  // Only return formatted text fields.
  return array_keys(array_filter(field_info_instances($entity_type, $bundle), function ($definition) use ($fields) {
    return in_array($fields[$definition['field_name']]['type'], array('text', 'text_long', 'text_with_summary'), TRUE);
  }));
}

/**
 * Parse an HTML snippet for any linked file with data-entity-id attributes.
 *
 * @param string $text
 *   The partial (X)HTML snippet to load. Invalid markup will be corrected on
 *   import.
 *
 * @return array
 *   An array of all found FIDs.
 */
function _editor_parse_file_fids($text) {
  $fids = array();

  if (stristr($text, 'data-entity-type="file"') !== FALSE) {
    $dom = filter_dom_load($text);
    $xpath = new DOMXPath($dom);

    foreach ($xpath->query('//*[@data-entity-type="file" and (@data-entity-uuid or @data-entity-id)]') as $node) {
      $uuid = $node->getAttribute('data-entity-uuid');
      $id = $node->getAttribute('data-entity-id');

      if (!empty($uuid) && module_exists('uuid')) {
        $ids = entity_get_id_by_uuid('file', [$uuid]);
        $fid = reset($ids);
      }
      else {
        $fid = $id;
      }

      $fids[] = $fid;
    }
  }

  return $fids;
}
