<?php

/**
 * @file
 * CKEditor 4 plugin API.
 */

/**
 * Retrieves the full list of installed CKEditor plugins.
 *
 * @return array
 *   An associative array of CKEditor plugins keyed by the internal name of the
 *   plugin. Each plugin may contain the following elements (all are optional
 *   except as noted):
 *   - file: Required for all external plugins. String file name of the plugin
 *     in the "location" directory.
 *   - plugin: The Drupal root-relative file path to the plugin JavaScript file.
 *   - internal: Boolean value indicating if the plugin is part of the
 *     compressed CKEditor library package and already loaded on all instances.
 *   - css: An array of CSS files that should be added by CKEditor. These files
 *     are used only when CKEditor is using an iframe wrapper around its
 *     content.
 *   - enabled callback: String containing a function name that can determine if
 *     this plugin should be enabled based on the current editor configuration.
 *   - config callback: String containing a function name that returns an array
 *     of CKEditor configuration settings.
 *   - settings form: Returns a settings form to configure this CKEditor plugin.
 *   - buttons: An array of buttons that are provided by this plugin. Each
 *     button is keyed by its CKEditor button name, and contains an array of
 *     button properties, including:
 *     - label: A human-readable, translated button name.
 *     - image: An image for the button to be used in the toolbar.
 *     - image_rtl: If the image needs to have a right-to-left version,
 *       specifies an alternative file that will be used in RTL editors.
 *     - image_alternative: If the button does not render as an image, specifies
 *       an HTML string representing the contents of the button.
 *     - attributes: An array of HTML attributes which are added to the button
 *       when it is rendered in the administrative section for assembling the
 *       toolbar.
 *     - multiple: Boolean value indicating if the button may be added multiple
 *       times to the toolbar.
 *     - required_html: If the button requires certain HTML tags to be allowed,
 *       specifies an array of tags.
 *
 * @see hook_editor_ckeditor_plugins()
 * @see hook_editor_ckeditor_plugins_alter()
 * @see hook_editor_ckeditor_PLUGIN_plugin_check()
 */
function editor_ckeditor_plugins() {
  $plugins = &drupal_static(__FUNCTION__);

  if (!isset($plugins)) {
    $plugins = module_invoke_all('editor_ckeditor_plugins');
    drupal_alter('editor_ckeditor_plugins', $plugins);

    // Load plugin files if specified.
    foreach ($plugins as $plugin_id => $plugin) {
      if (!empty($plugin['file'])) {
        if (file_exists($plugin['file'])) {
          require_once DRUPAL_ROOT . '/' . $plugin['file'];
        }
      }
    }
  }

  return $plugins;
}

/**
 * Retrieves enabled plugins' files, keyed by plugin ID.
 *
 * For CKEditor plugins that contain:
 *  - 'buttons', not 'contextual', a plugin is enabled if at least one of its
 *    buttons is in the toolbar.
 *  - 'contextual', not 'buttons', a plugin is enabled if its 'enabled callback'
 *    returns TRUE.
 *  - In both scenarios, a plugin is enabled if either is the case.
 *
 * Internal plugins (those that are part of the bundled build of CKEditor) are
 * excluded by default, since they are loaded implicitly. If you need to know
 * even implicitly loaded (i.e. internal) plugins, then set the optional
 * second parameter.
 *
 * @param $format
 *   An object representing a text format.
 * @param bool $include_internal_plugins
 *   Defaults to FALSE. When set to TRUE, plugins whose isInternal() method
 *   returns TRUE will also be included.
 *
 * @return array
 *   A list of the enabled CKEditor plugins, with the plugin IDs as keys and
 *   the Drupal root-relative plugin files as values.
 *   For internal plugins, the value is NULL.
 */
function editor_ckeditor_get_enabled_plugin_files($format, $include_internal_plugins = FALSE) {
  $plugins = editor_ckeditor_plugins();
  $toolbar_buttons = editor_ckeditor_get_enabled_buttons($format);
  $enabled_plugins = array();
  $additional_plugins = array();

  foreach ($plugins as $plugin_id => $plugin) {
    $plugin += array(
      'internal' => FALSE,
      'dependencies' => array(),
      'libraries' => array(),
      'buttons' => array(),
      'contextual' => FALSE,
      'plugin' => FALSE,
    );

    if (!$include_internal_plugins && !empty($plugin['internal'])) {
      continue;
    }

    $enabled = FALSE;

    // Enable this plugin if it provides a button that has been enabled.
    if (!empty($plugin['buttons'])) {
      $plugin_buttons = array_keys($plugin['buttons']);
      $enabled = (count(array_intersect($toolbar_buttons, $plugin_buttons)) > 0);
    }

    // Otherwise enable this plugin if it declares itself as enabled.
    if (!$enabled && !empty($plugin['contextual'])) {
      $enabled = $plugin['enabled callback']($format, $plugin_id);
    }

    if ($enabled) {
      $enabled_plugins[$plugin_id] = !empty($plugin['internal']) ? NULL : $plugin['plugin'];
      // Check if this plugin has dependencies that also need to be enabled.
      $additional_plugins = array_merge($additional_plugins, array_diff($plugin['dependencies'], $additional_plugins));
    }
  }

  // Add the list of dependent plugins.
  foreach ($additional_plugins as $plugin_id => $plugin) {
    $plugin = $additional_plugins[$plugin_id];
    $enabled_plugins[$plugin_id] = !empty($plugin['internal']) ? NULL : $plugin['plugin'];
  }

  // Always return plugins in the same order.
  asort($enabled_plugins);

  return $enabled_plugins;
}

/**
 * Gets the enabled toolbar buttons in the given text editor instance.
 *
 * @param $format
 *   An object representing a text format.
 *
 * @return string|array()
 *   A list of the toolbar buttons enabled in the given text editor instance.
 */
function editor_ckeditor_get_enabled_buttons($format) {
  $toolbar_rows = array();
  $settings = $format->editor_settings;
  foreach ($settings['toolbar'] as $row_number => $row) {
    foreach ($row as $button_group) {
      $toolbar_rows[] = array_reduce($settings['toolbar'][$row_number], function ($result, $button_group) { // could be $settings['toolbar'][$row]?
        return array_merge($result, $button_group['items']);
      }, array());
    }
  }
  return array_unique(drupal_array_merge_deep_array($toolbar_rows));
}

/**
 * Retrieves all available CKEditor buttons, keyed by plugin ID.
 *
 * @return array
 *   All available CKEditor buttons, with plugin IDs as keys and button
 *   metadata (as implemented by getButtons()) as values.
 */
function editor_ckeditor_get_buttons() {
  $plugins = editor_ckeditor_plugins();
  $buttons_plugins = array();

  foreach (array_keys($plugins) as $plugin_id) {
    $plugin = $plugins[$plugin_id];
    if (!empty($plugin['buttons'])) {
      $buttons_plugins[$plugin_id] = $plugin['buttons'];
    }
  }

  return $buttons_plugins;
}
