<?php

/**
 * @file
 * Defines the "internal" plugin (i.e. core plugins part of our CKEditor build).
 */

/**
 * Enabled callback for the 'internal' plugin.
 *
 * @param object $format
 *   A text format object.
 *
 * @return bool
 *   TRUE if the plugin is enabled, FALSE otherwise.
 */
function editor_ckeditor_internal_plugin_check($format) {
  // This plugin represents the core CKEditor plugins. They're always enabled:
  // its configuration is always necessary.
  return TRUE;
}

/**
 * Config for the "internal" CKEditor plugin.
 *
 * @param object $format
 *  A text format object.
 *
 * @return array
 *   An associative array of configuration settings.
 */
function editor_ckeditor_internal_config($format) {
  // Reasonable defaults that provide expected basic behavior.
  $config = array(
    // Don't load CKEditor's config.js file.
    'customConfig' => '',
    'pasteFromWordPromptCleanup' => TRUE,
    'resize_dir' => 'vertical',
    'justifyClasses' => array('text-align-left', 'text-align-center', 'text-align-right', 'text-align-justify'),
    'entities' => FALSE,
    'disableNativeSpellChecker' => FALSE,
    'indentClasses' => array('indent1', 'indent2', 'indent3'),
    'coreStyles_strike' => array('element' => 'del'),
    'coreStyles_underline' => array('element' => 'span', 'attributes' => array('class' => 'underline')),
  );

  // Add the allowedContent setting, which ensures CKEditor only allows tags
  // and attributes that are allowed by the text format for this text editor.
  list($config['allowedContent'], $config['disallowedContent']) = editor_ckeditor_generate_acf_settings($format);

  // Add the format_tags setting, if its button is enabled.
  $toolbar_buttons = editor_ckeditor_get_enabled_buttons($format);
  if (in_array('Format', $toolbar_buttons)) {
    $config['format_tags'] = editor_ckeditor_generate_format_tags_setting($format);
  }

  return $config;
}

/**
 * Builds the "format_tags" configuration part of the CKEditor JS settings.
 *
 * @param $format
 *  An object representing a text format.
 *
 * @return array
 *   An array containing the "format_tags" configuration.
 */
function editor_ckeditor_generate_format_tags_setting($format) {
  // When no editor is associated yet, assume no tag is allowed.
  if (empty($format->editor)) {
    return array();
  }

  $cid = $format->format . ':editor_ckeditor_internal_format_tags';

  if ($cached = cache_get($cid, 'cache_filter')) {
    $format_tags = $cached->data;
  }
  else {
    // The <p> tag is always allowed — HTML without <p> tags is nonsensical.
    $format_tags = array('p');

    // Given the list of possible format tags, automatically determine whether
    // the current text format allows this tag, and thus whether it should show
    // up in the "Format" dropdown.
    $possible_format_tags = array('h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'pre');
    foreach ($possible_format_tags as $tag) {
      $input = '<' . $tag . '>TEST</' . $tag . '>';
      $output = trim(check_markup($input, $format->format));
      if (filter_dom_load($output)->getElementsByTagName($tag)->length !== 0) {
        $format_tags[] = $tag;
      }
    }
    $format_tags = implode(';', $format_tags);

    // Cache the "format_tags" configuration. This cache item is infinitely
    // valid; it only changes whenever the text format is changed, hence it's
    // stored in the text format's cache bin.
    cache_set($cid, $format_tags, 'cache_filter');
  }

  return $format_tags;
}

/**
 * Builds the ACF part of the CKEditor JS settings.
 *
 * This ensures that CKEditor obeys the HTML restrictions defined by Drupal's
 * filter system, by enabling CKEditor's Advanced Content Filter (ACF)
 * functionality: http://ckeditor.com/blog/CKEditor-4.1-RC-Released.
 *
 * @param object $format
 *   The text format object.
 *
 * @return array
 *   An array with two values:
 *   - the first value is the "allowedContent" setting: a well-formatted array
 *     or TRUE. The latter indicates that anything is allowed.
 *   - the second value is the "disallowedContent" setting: a well-formatted
 *     array or FALSE. The latter indicates that nothing is disallowed.
 */
function editor_ckeditor_generate_acf_settings($format) {
  // When no text format is associated yet, assume nothing is disallowed, so
  // set allowedContent to true.
  if (!$format->editor) {
    return TRUE;
  }

  $filter_types = editor_get_filter_types($format);

  // When nothing is disallowed, set allowedContent to true.
  if (!in_array(TYPE_HTML_RESTRICTOR, $filter_types)) {
    return array(TRUE, FALSE);
  }
  // Generate setting that accurately reflects allowed tags and attributes.
  else {
    $get_attribute_values = function ($attribute_values, $allowed_values) {
      $values = array_keys(array_filter($attribute_values, function ($value) use ($allowed_values) {
        if ($allowed_values) {
          return $value !== FALSE;
        }
        else {
          return $value === FALSE;
        }
      }));
      if (count($values)) {
        return implode(',', $values);
      }
      else {
        return NULL;
      }
    };

    $html_restrictions = editor_format_get_html_restrictions($format);
    // When all HTML is allowed, also set allowedContent to true and
    // disallowedContent to false.
    if ($html_restrictions === TRUE) {
      return array(TRUE, FALSE);
    }

    $allowed = array();
    $disallowed = array();
    if (isset($html_restrictions['forbidden_tags'])) {
      foreach ($html_restrictions['forbidden_tags'] as $tag) {
        $disallowed[$tag] = TRUE;
      }
    }

    foreach ($html_restrictions['allowed'] as $tag => $attributes) {
      // Tell CKEditor the tag is allowed, but no attributes.
      if ($attributes === FALSE) {
        $allowed[$tag] = [
          'attributes' => FALSE,
          'styles' => FALSE,
          'classes' => FALSE,
        ];
      }
      // Tell CKEditor the tag is allowed, as well as any attribute on it. The
      // "style" and "class" attributes are handled separately by CKEditor:
      // they are disallowed even if you specify it in the list of allowed
      // attributes, unless you state specific values for them that are
      // allowed. Or, in this case: any value for them is allowed.
      elseif ($attributes === TRUE) {
        $allowed[$tag] = [
          'attributes' => TRUE,
          'styles' => TRUE,
          'classes' => TRUE,
        ];

        // We've just marked that any value for the "style" and "class"
        // attributes is allowed. However, that may not be the case: the "*"
        // tag may still apply restrictions.
        // Since CKEditor's ACF follows the following principle:
        // "Once validated, an element or its property cannot be invalidated by
        // another rule."
        // That means that the most permissive setting wins. Which means that
        // it will still be allowed by CKEditor to e.g. define any style, no
        // matter what the "*" tag's restrictions may be. If there's a setting
        // for either the "style" or "class" attribute, it cannot possibly be
        // more permissive than what was set above. Hence: inherit from the
        // "*" tag where possible.
        if (isset($html_restrictions['allowed']['*'])) {
          $wildcard = $html_restrictions['allowed']['*'];

          if (isset($wildcard['style'])) {
            if (!is_array($wildcard['style'])) {
              $allowed[$tag]['styles'] = $wildcard['style'];
            }
            else {
              $allowed_styles = $get_attribute_values($wildcard['style'], TRUE);

              if (isset($allowed_styles)) {
                $allowed[$tag]['styles'] = $allowed_styles;
              }
              else {
                unset($allowed[$tag]['styles']);
              }
            }
          }

          if (isset($wildcard['class'])) {
            if (!is_array($wildcard['class'])) {
              $allowed[$tag]['classes'] = $wildcard['class'];
            }
            else {
              $allowed_classes = $get_attribute_values($wildcard['class'], TRUE);

              if (isset($allowed_classes)) {
                $allowed[$tag]['classes'] = $allowed_classes;
              }
              else {
                unset($allowed[$tag]['classes']);
              }
            }
          }
        }
      }
      // Tell CKEditor the tag is allowed, along with some tags.
      elseif (is_array($attributes)) {
        // Set defaults (these will be overridden below if more specific
        // values are present).
        $allowed[$tag] = array(
          'attributes' => FALSE,
          'styles' => FALSE,
          'classes' => FALSE,
        );
        // Configure allowed attributes, allowed "style" attribute values and
        // allowed "class" attribute values.
        // CKEditor only allows specific values for the "class" and "style"
        // attributes; so ignore restrictions on other attributes, which
        // Drupal filters may provide.
        // NOTE: A Drupal contrib module can subclass this class, override the
        // getConfig() method, and override the JavaScript at
        // Drupal.editors.ckeditor to somehow make validation of values for
        // attributes other than "class" and "style" work.
        $allowed_attributes = array_filter($attributes, function ($value) {
          return $value !== FALSE;
        });
        if (count($allowed_attributes)) {
          $allowed[$tag]['attributes'] = implode(',', array_keys($allowed_attributes));
        }
        if (isset($allowed_attributes['style'])) {
          if (is_bool($allowed_attributes['style'])) {
            $allowed[$tag]['styles'] = $allowed_attributes['style'];
          }
          elseif (is_array($allowed_attributes['style'])) {
            $allowed_classes = $get_attribute_values($allowed_attributes['style'], TRUE);
            if (isset($allowed_classes)) {
              $allowed[$tag]['styles'] = $allowed_classes;
            }
          }
        }
        if (isset($allowed_attributes['class'])) {
          if (is_bool($allowed_attributes['class'])) {
            $allowed[$tag]['classes'] = $allowed_attributes['class'];
          }
          elseif (is_array($allowed_attributes['class'])) {
            $allowed_classes = $get_attribute_values($allowed_attributes['class'], TRUE);
            if (isset($allowed_classes)) {
              $allowed[$tag]['classes'] = $allowed_classes;
            }
          }
        }

        // Handle disallowed attributes analogously. However, to handle *dis-
        // allowed* attribute values, we must look at *allowed* attributes'
        // disallowed attribute values! After all, a disallowed attribute
        // implies that all of its possible attribute values are disallowed,
        // thus we must look at the disallowed attribute values on allowed
        // attributes.
        $disallowed_attributes = array_filter($attributes, function ($value) {
          return $value === FALSE;
        });

        if (count($disallowed_attributes)) {
          // No need to blacklist the 'class' or 'style' attributes; CKEditor
          // handles them separately (if no specific class or style attribute
          // values are allowed, then those attributes are disallowed).
          if (isset($disallowed_attributes['class'])) {
            unset($disallowed_attributes['class']);
          }

          if (isset($disallowed_attributes['style'])) {
            unset($disallowed_attributes['style']);
          }

          $disallowed[$tag]['attributes'] = implode(',', array_keys($disallowed_attributes));
        }

        if (isset($allowed_attributes['style']) && is_array($allowed_attributes['style'])) {
          $disallowed_styles = $get_attribute_values($allowed_attributes['style'], FALSE);

          if (isset($disallowed_styles)) {
            $disallowed[$tag]['styles'] = $disallowed_styles;
          }
        }

        if (isset($allowed_attributes['class']) && is_array($allowed_attributes['class'])) {
          $disallowed_classes = $get_attribute_values($allowed_attributes['class'], FALSE);

          if (isset($disallowed_classes)) {
            $disallowed[$tag]['classes'] = $disallowed_classes;
          }
        }
      }
    }

    ksort($allowed);
    ksort($disallowed);

    return array($allowed, $disallowed);
  }
}
